using UnityEngine;
using HMLLibrary;
using HarmonyLib;
using System.Collections.Generic;
using System.Reflection;
using System.Reflection.Emit;
using System;

public class InventoryHotkeys : Mod
{
    Harmony harmony;
    string harmonyId = "fynikoto.inventoryHotkeys";

    public void Start()
    {
        harmony = new Harmony(harmonyId);
        harmony.PatchAll(Assembly.GetExecutingAssembly());
        Debug.Log("Mod InventoryHotkeys has been loaded!");
    }

    public void Update() {
        if (Input.GetKey(KeyCode.LeftAlt)) {
            // Debug.Log("Alt held down");
        }
    }

    public void OnModUnload()
    {
        harmony.UnpatchAll(harmonyId);
        Debug.Log("Mod InventoryHotkeys has been unloaded!");
    }
}

[HarmonyPatch(typeof(Inventory), "MoveItem")]
class InventoryPatch {
    public static IEnumerable<CodeInstruction> Transpiler(IEnumerable<CodeInstruction> instructions) {
        MethodInfo GetKey = typeof(Input).GetMethod("GetKey", BindingFlags.Public | BindingFlags.Static, null, new Type[] { typeof(KeyCode) }, null);


        var codes = new List<CodeInstruction>(instructions);
        
        int positionOfSwitch = GetPositionOfOpCode(codes, OpCodes.Switch);
        int[] positionOfLabels = GetFirstPositionsOfLabels(codes, codes[positionOfSwitch].operand as Label[]);

        if (positionOfSwitch < 0 || positionOfLabels[0] < 0 || positionOfLabels[2] < 0) {
            Debug.LogError("[InventoryHotkeys] Something went wrong! MoveItem could not be patched!");
            return codes;
        }

        Label labelofLMB = codes[positionOfLabels[0]].labels[0];
        Label labelofMMB = codes[positionOfLabels[2]].labels[0];
        codes[positionOfLabels[0]].labels.Remove(labelofLMB);

        var useAltToHalfStacks = new List<CodeInstruction>() {
            new CodeInstruction(OpCodes.Ldc_I4, (int)KeyCode.LeftAlt),
            new CodeInstruction(OpCodes.Call, GetKey),
            new CodeInstruction(OpCodes.Brtrue, labelofMMB)
        };

        useAltToHalfStacks[0].labels.Add(labelofLMB);

        codes.InsertRange(positionOfLabels[0], useAltToHalfStacks);

        return codes;
    }

    private static int GetPositionOfOpCode(List<CodeInstruction> codes, OpCode oc) {
        for (int i = 0; i < codes.Count; i++) {
            if (codes[i].opcode == oc) {
                return i;
            }
        }
        return -1;
    }

    private static int[] GetFirstPositionsOfLabels(List<CodeInstruction> codes, Label[] labels) {
        int[] positions = new int[labels.Length];

        for (int i = 0; i < codes.Count; i++) {
            for(int l = 0; l < labels.Length; l++) {
                if (codes[i].labels.Contains(labels[l]) && positions[l] == 0) {
                    positions[l] = i;
                }
            }
        }
        return positions;
    }
}